'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.tail = exports.head = exports.slice = exports.add = exports.extract = exports.unsplats = exports.splats = exports.falcy = exports.makeType = exports.liftA3 = exports.liftA2 = exports.ap = exports.contains = exports.chain = exports.filter = exports.reduce = exports.of = exports.propEq = exports.hasProp = exports.prop = exports.complement = exports.condition = exports.tap = exports.push = exports.mcompose = exports.compose = exports.id = exports.concat = exports.isDefined = exports.isArray = exports.isObject = undefined;

var _curry = require('./core/curry2');

var _curry2 = _interopRequireDefault(_curry);

var _curry3 = require('./core/curry3');

var _curry4 = _interopRequireDefault(_curry3);

var _map = require('./map');

var _map2 = _interopRequireDefault(_map);

var _lodash = require('lodash.isplainobject');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var isObject = exports.isObject = _lodash2.default;
var isArray = exports.isArray = function isArray(v) {
  return Object.prototype.toString.call(v) === '[object Array]';
};
var isDefined = exports.isDefined = function isDefined(v) {
  return v === undefined;
};
/**
 * [a] -> [a] -> [a]
 */
var concat = exports.concat = (0, _curry2.default)(function (a, b) {
  return a.concat(b);
});

/**
 * a -> a
 */
var id = exports.id = function id(v) {
  return v;
};

var compose = exports.compose = function compose() {
  for (var _len = arguments.length, fns = Array(_len), _key = 0; _key < _len; _key++) {
    fns[_key] = arguments[_key];
  }

  return function (v) {
    return fns.reduceRight(function (carry, fn) {
      return fn(carry);
    }, v);
  };
};
var mcompose = exports.mcompose = function mcompose() {
  for (var _len2 = arguments.length, fns = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    fns[_key2] = arguments[_key2];
  }

  return function (u) {
    return fns.reduceRight(function (v, f) {
      return (0, _map2.default)(f, v);
    }, u);
  };
};

var push = exports.push = (0, _curry2.default)(function (ls, v) {
  ls.push(v);
  return v;
});

// use tap for debugger and console.log
var tap = exports.tap = (0, _curry4.default)(function (f, g, v) {
  f(g(v));
  return v;
});

var condition = exports.condition = (0, _curry4.default)(function (predicate, trueFn, falseFn) {
  return function (v) {
    return predicate(v) ? trueFn(v) : falseFn(v);
  };
});

var complement = exports.complement = function complement(fn) {
  return function () {
    for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args[_key3] = arguments[_key3];
    }

    return !fn.apply(null, args);
  };
};

var prop = exports.prop = (0, _curry2.default)(function (prop, obj) {
  return obj[prop];
});
var hasProp = exports.hasProp = (0, _curry2.default)(function (propName, obj) {
  return typeof prop(propName, obj) !== 'undefined';
});
var propEq = exports.propEq = (0, _curry4.default)(function (propName, v, obj) {
  return prop(propName, obj) === v;
});

var of = exports.of = function of(x) {
  return x.of;
};

var reduce = exports.reduce = (0, _curry4.default)(function (acc, init, xs) {
  return xs.reduce(acc, init);
});
var filter = exports.filter = (0, _curry2.default)(function (f, u) {
  return u.filter(function (v) {
    return f(v);
  });
});
var chain = exports.chain = (0, _curry2.default)(function (f, m) {
  return m.chain(f);
});

// xs -> any -> Boolean
var contains = exports.contains = (0, _curry2.default)(function (xs, key) {
  return xs.indexOf(key) !== -1;
});

var ap = exports.ap = (0, _curry2.default)(function (u, v) {
  return u.ap(v);
});
var liftA2 = exports.liftA2 = (0, _curry4.default)(function (f, x, y) {
  return x.map(f).ap(y);
});
var liftA3 = exports.liftA3 = (0, _curry4.default)(function (f, x, y, z) {
  return x.map(f).ap(y).ap(z);
});

var makeType = exports.makeType = function makeType(type, v) {
  return new type(v);
};

var falcy = exports.falcy = function falcy(v) {
  return v == null || v !== v;
};

var splats = exports.splats = function splats(fn) {
  return function (arr) {
    return fn.apply(undefined, _toConsumableArray(arr));
  };
};
var unsplats = exports.unsplats = function unsplats(fn) {
  return function () {
    for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
      args[_key4] = arguments[_key4];
    }

    return fn(args);
  };
};
var extract = exports.extract = function extract(v) {
  return v.extract();
};

var add = exports.add = (0, _curry2.default)(function (a, b) {
  return a + b;
});

var slice = exports.slice = (0, _curry4.default)(function (start, end, xs) {
  return xs.slice(start, end);
});
var head = exports.head = prop(0);
var tail = exports.tail = slice(1, Infinity);